@@ -1,5 +1,5 @@
 /* Lunzip - Decompressor for the lzip format
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,17 +52,15 @@ int list_files( const char * const filenames[], const int num_filenames,
   bool stdin_used = false;
   for( i = 0; i < num_filenames; ++i )
     {
-    const char * input_filename;
-    struct Lzip_index lzip_index;
-    struct stat in_stats;				/* not used */
-    int infd;
     const bool from_stdin = ( strcmp( filenames[i], "-" ) == 0 );
     if( from_stdin ) { if( stdin_used ) continue; else stdin_used = true; }
-    input_filename = from_stdin ? "(stdin)" : filenames[i];
-    infd = from_stdin ? STDIN_FILENO :
+    const char * const input_filename = from_stdin ? "(stdin)" : filenames[i];
+    struct stat in_stats;				/* not used */
+    const int infd = from_stdin ? STDIN_FILENO :
       open_instream( input_filename, &in_stats, false, true );
     if( infd < 0 ) { set_retval( &retval, 1 ); continue; }
 
+    struct Lzip_index lzip_index;
     Li_init( &lzip_index, infd, ignore_trailing, loose_trailing );
     close( infd );
     if( lzip_index.retval != 0 )
@@ -71,37 +69,36 @@ int list_files( const char * const filenames[], const int num_filenames,
       set_retval( &retval, lzip_index.retval );
       Li_free( &lzip_index ); continue;
       }
-    if( verbosity >= 0 )
+    if( verbosity < 0 ) { Li_free( &lzip_index ); continue; }
+    const unsigned long long udata_size = Li_udata_size( &lzip_index );
+    const unsigned long long cdata_size = Li_cdata_size( &lzip_index );
+    total_comp += cdata_size; total_uncomp += udata_size; ++files;
+    const long members = lzip_index.members;
+    if( first_post )
       {
-      const unsigned long long udata_size = Li_udata_size( &lzip_index );
-      const unsigned long long cdata_size = Li_cdata_size( &lzip_index );
-      total_comp += cdata_size; total_uncomp += udata_size; ++files;
-      if( first_post )
-        {
-        first_post = false;
-        if( verbosity >= 1 ) fputs( "   dict   memb  trail ", stdout );
-        fputs( "  uncompressed     compressed   saved  name\n", stdout );
-        }
-      if( verbosity >= 1 )
-        printf( "%s %5ld %6lld ", format_ds( lzip_index.dictionary_size ),
-                lzip_index.members, Li_file_size( &lzip_index ) - cdata_size );
-      list_line( udata_size, cdata_size, input_filename );
-
-      if( verbosity >= 2 && lzip_index.members > 1 )
-        {
-        long i;
-        fputs( " member      data_pos      data_size     member_pos    member_size\n", stdout );
-        for( i = 0; i < lzip_index.members; ++i )
-          {
-          const struct Block * db = Li_dblock( &lzip_index, i );
-          const struct Block * mb = Li_mblock( &lzip_index, i );
-          printf( "%6ld %14llu %14llu %14llu %14llu\n",
-                  i + 1, db->pos, db->size, mb->pos, mb->size );
-          }
-        first_post = true;	/* reprint heading after list of members */
-        }
-      fflush( stdout );
+      first_post = false;
+      if( verbosity >= 1 ) fputs( "   dict   memb  trail ", stdout );
+      fputs( "  uncompressed     compressed   saved  name\n", stdout );
       }
+    if( verbosity >= 1 )
+      printf( "%s %5ld %6lld ", format_ds( lzip_index.dictionary_size ),
+              members, Li_file_size( &lzip_index ) - cdata_size );
+    list_line( udata_size, cdata_size, input_filename );
+
+    if( verbosity >= 2 && members > 1 )
+      {
+      long i;
+      fputs( " member      data_pos      data_size     member_pos    member_size\n", stdout );
+      for( i = 0; i < members; ++i )
+        {
+        const struct Block * db = Li_dblock( &lzip_index, i );
+        const struct Block * mb = Li_mblock( &lzip_index, i );
+        printf( "%6ld %14llu %14llu %14llu %14llu\n",
+                i + 1, db->pos, db->size, mb->pos, mb->size );
+        }
+      first_post = true;	/* reprint heading after list of members */
+      }
+    fflush( stdout );
     Li_free( &lzip_index );
     }
   if( verbosity >= 0 && files > 1 )
