@@ -1,5 +1,5 @@
 /* Lunzip - Decompressor for the lzip format
-   Copyright (C) 2010-2021 Antonio Diaz Diaz.
+   Copyright (C) 2010-2022 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
    Exit status: 0 for a normal exit, 1 for environmental problems
    (file not found, invalid flags, I/O errors, etc), 2 to indicate a
    corrupt or invalid input file, 3 for an internal consistency error
-   (eg, bug) which caused lunzip to panic.
+   (e.g., bug) which caused lunzip to panic.
 */
 
 #define _FILE_OFFSET_BITS 64
@@ -36,9 +36,9 @@
 #include <unistd.h>
 #include <utime.h>
 #include <sys/stat.h>
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
 #include <io.h>
-#if defined(__MSVCRT__)
+#if defined __MSVCRT__
 #define fchmod(x,y) 0
 #define fchown(x,y,z) 0
 #define SIGHUP SIGTERM
@@ -50,7 +50,7 @@
 #define S_IWOTH 0
 #endif
 #endif
-#if defined(__DJGPP__)
+#if defined __DJGPP__
 #define S_ISSOCK(x) 0
 #define S_ISVTX 0
 #endif
@@ -68,10 +68,15 @@
 #error "Environments where CHAR_BIT != 8 are not supported."
 #endif
 
+#if ( defined  SIZE_MAX &&  SIZE_MAX < UINT_MAX ) || \
+    ( defined SSIZE_MAX && SSIZE_MAX <  INT_MAX )
+#error "Environments where 'size_t' is narrower than 'int' are not supported."
+#endif
+
 int verbosity = 0;
 
 static const char * const program_name = "lunzip";
-static const char * const program_year = "2021";
+static const char * const program_year = "2022";
 static const char * invocation_name = "lunzip";		/* default value */
 
 static const struct { const char * from; const char * to; } known_extensions[] = {
@@ -130,7 +135,7 @@ static void show_help( void )
           "'tar -xf foo.tar.lz' or 'lunzip -cd foo.tar.lz | tar -xf -'.\n"
           "\nExit status: 0 for a normal exit, 1 for environmental problems (file\n"
           "not found, invalid flags, I/O errors, etc), 2 to indicate a corrupt or\n"
-          "invalid input file, 3 for an internal consistency error (eg, bug) which\n"
+          "invalid input file, 3 for an internal consistency error (e.g., bug) which\n"
           "caused lunzip to panic.\n"
           "\nThe ideas embodied in lunzip are due to (at least) the following people:\n"
           "Abraham Lempel and Jacob Ziv (for the LZ algorithm), Andrey Markov (for the\n"
@@ -174,8 +179,6 @@ struct Pretty_print
 static void Pp_init( struct Pretty_print * const pp,
                      const char * const filenames[], const int num_filenames )
   {
-  unsigned stdin_name_len;
-  int i;
   pp->name = 0;
   pp->padded_name = 0;
   pp->stdin_name = "(stdin)";
@@ -183,7 +186,8 @@ static void Pp_init( struct Pretty_print * const pp,
   pp->first_post = false;
 
   if( verbosity <= 0 ) return;
-  stdin_name_len = strlen( pp->stdin_name );
+  const unsigned stdin_name_len = strlen( pp->stdin_name );
+  int i;
   for( i = 0; i < num_filenames; ++i )
     {
     const char * const s = filenames[i];
@@ -217,16 +221,14 @@ static void Pp_reset( struct Pretty_print * const pp )
 
 void Pp_show_msg( struct Pretty_print * const pp, const char * const msg )
   {
-  if( verbosity >= 0 )
+  if( verbosity < 0 ) return;
+  if( pp->first_post )
     {
-    if( pp->first_post )
-      {
-      pp->first_post = false;
-      fputs( pp->padded_name, stderr );
-      if( !msg ) fflush( stderr );
-      }
-    if( msg ) fprintf( stderr, "%s\n", msg );
+    pp->first_post = false;
+    fputs( pp->padded_name, stderr );
+    if( !msg ) fflush( stderr );
     }
+  if( msg ) fprintf( stderr, "%s\n", msg );
   }
 
 
@@ -264,17 +266,53 @@ void show_header( const unsigned dictionary_size )
   }
 
 
-static unsigned long getnum( const char * const ptr,
+/* separate large numbers >= 100_000 in groups of 3 digits using '_' */
+static const char * format_num3( unsigned long long num )
+  {
+  const char * const si_prefix = "kMGTPEZY";
+  const char * const binary_prefix = "KMGTPEZY";
+  enum { buffers = 8, bufsize = 4 * sizeof (long long) };
+  static char buffer[buffers][bufsize];	/* circle of static buffers for printf */
+  static int current = 0;
+  int i;
+  char * const buf = buffer[current++]; current %= buffers;
+  char * p = buf + bufsize - 1;		/* fill the buffer backwards */
+  *p = 0;	/* terminator */
+  if( num > 1024 )
+    {
+    char prefix = 0;			/* try binary first, then si */
+    for( i = 0; i < 8 && num >= 1024 && num % 1024 == 0; ++i )
+      { num /= 1024; prefix = binary_prefix[i]; }
+    if( prefix ) *(--p) = 'i';
+    else
+      for( i = 0; i < 8 && num >= 1000 && num % 1000 == 0; ++i )
+        { num /= 1000; prefix = si_prefix[i]; }
+    if( prefix ) *(--p) = prefix;
+    }
+  const bool split = num >= 100000;
+
+  for( i = 0; ; )
+    {
+    *(--p) = num % 10 + '0'; num /= 10; if( num == 0 ) break;
+    if( split && ++i >= 3 ) { i = 0; *(--p) = '_'; }
+    }
+  return p;
+  }
+
+
+static unsigned long getnum( const char * const arg,
+                             const char * const option_name,
                              const unsigned long llimit,
                              const unsigned long ulimit )
   {
-  unsigned long result;
   char * tail;
   errno = 0;
-  result = strtoul( ptr, &tail, 0 );
-  if( tail == ptr )
+  unsigned long long result = strtoul( arg, &tail, 0 );
+  if( tail == arg )
     {
-    show_error( "Bad or missing numerical argument.", 0, true );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Bad or missing numerical argument in "
+               "option '%s'.\n", program_name, option_name );
     exit( 1 );
     }
 
@@ -297,7 +335,9 @@ static unsigned long getnum( const char * const ptr,
       }
     if( exponent <= 0 )
       {
-      show_error( "Bad multiplier in numerical argument.", 0, true );
+      if( verbosity >= 0 )
+        fprintf( stderr, "%s: Bad multiplier in numerical argument of "
+                 "option '%s'.\n", program_name, option_name );
       exit( 1 );
       }
     for( i = 0; i < exponent; ++i )
@@ -309,21 +349,24 @@ static unsigned long getnum( const char * const ptr,
   if( !errno && ( result < llimit || result > ulimit ) ) errno = ERANGE;
   if( errno )
     {
-    show_error( "Numerical argument out of limits.", 0, false );
+    if( verbosity >= 0 )
+      fprintf( stderr, "%s: Numerical argument out of limits [%s,%s] "
+               "in option '%s'.\n", program_name, format_num3( llimit ),
+               format_num3( ulimit ), option_name );
     exit( 1 );
     }
   return result;
   }
 
 
-static int get_dict_size( const char * const arg )
+static int get_dict_size( const char * const arg, const char * const option_name )
   {
   char * tail;
   const long bits = strtol( arg, &tail, 0 );
   if( bits >= min_dictionary_bits &&
       bits <= max_dictionary_bits && *tail == 0 )
     return 1 << bits;
-  return getnum( arg, min_dictionary_size, max_dictionary_size );
+  return getnum( arg, option_name, min_dictionary_size, max_dictionary_size );
   }
 
 
@@ -468,7 +511,7 @@ static bool check_tty_in( const char * const input_filename, const int infd,
   if( isatty( infd ) )				/* for example /dev/tty */
     { show_file_error( input_filename,
                        "I won't read compressed data from a terminal.", 0 );
-      close( infd ); set_retval( retval, 1 );
+      close( infd ); set_retval( retval, 2 );
       if( program_mode != m_test ) cleanup_and_fail( *retval );
       return false; }
   return true;
@@ -559,12 +602,9 @@ static int decompress( const unsigned long long cfile_size, const int infd,
 
   for( first_member = true; ; first_member = false )
     {
-    int result, size;
-    unsigned dictionary_size;
     Lzip_header header;
-    struct LZ_decoder decoder;
     Rd_reset_member_position( &rdec );
-    size = Rd_read_data( &rdec, header, Lh_size );
+    const int size = Rd_read_data( &rdec, header, Lh_size );
     if( Rd_finished( &rdec ) )			/* End Of File */
       {
       if( first_member )
@@ -594,20 +634,21 @@ static int decompress( const unsigned long long cfile_size, const int infd,
     if( !Lh_verify_version( header ) )
       { Pp_show_msg( pp, bad_version( Lh_version( header ) ) );
         retval = 2; break; }
-    dictionary_size = Lh_get_dictionary_size( header );
+    const unsigned dictionary_size = Lh_get_dictionary_size( header );
     if( !isvalid_ds( dictionary_size ) )
       { Pp_show_msg( pp, bad_dict_msg ); retval = 2; break; }
 
     if( verbosity >= 2 || ( verbosity == 1 && first_member ) )
       Pp_show_msg( pp, 0 );
 
+    struct LZ_decoder decoder;
     if( !LZd_init( &decoder, &rdec, buffer_size, dictionary_size, outfd ) )
       {
       Pp_show_msg( pp, "Not enough memory. Try a smaller output buffer size." );
       retval = 1; break;
       }
     show_dprogress( cfile_size, partial_file_pos, &rdec, pp );	/* init */
-    result = LZd_decode_member( &decoder, pp );
+    const int result = LZd_decode_member( &decoder, pp );
     partial_file_pos += Rd_member_position( &rdec );
     LZd_free( &decoder );
     if( result != 0 )
@@ -696,23 +737,15 @@ void show_dprogress( const unsigned long long cfile_size,
 int main( const int argc, const char * const argv[] )
   {
   const char * default_output_filename = "";
-  static struct Arg_parser parser;	/* static because valgrind complains */
-  static struct Pretty_print pp;	/* and memory management in C sucks */
-  static const char ** filenames = 0;
-  int num_filenames = 0;
   unsigned buffer_size = max_dictionary_size;
   enum Mode program_mode = m_compress;
-  int argind = 0;
-  int failed_tests = 0;
-  int retval = 0;
   int i;
-  bool filenames_given = false;
   bool force = false;
   bool ignore_trailing = true;
   bool keep_input_files = false;
   bool loose_trailing = false;
-  bool stdin_used = false;
   bool to_stdout = false;
+  if( argc > 0 ) invocation_name = argv[0];
 
   enum { opt_lt = 256 };
   const struct ap_Option options[] =
@@ -734,19 +767,22 @@ int main( const int argc, const char * const argv[] )
     { opt_lt, "loose-trailing", ap_no  },
     {  0 , 0,                   ap_no  } };
 
-  if( argc > 0 ) invocation_name = argv[0];
   CRC32_init();
 
+  /* static because valgrind complains and memory management in C sucks */
+  static struct Arg_parser parser;
   if( !ap_init( &parser, argc, argv, options, 0 ) )
     { show_error( mem_msg, 0, false ); return 1; }
   if( ap_error( &parser ) )				/* bad option */
     { show_error( ap_error( &parser ), 0, true ); return 1; }
 
+  int argind = 0;
   for( ; argind < ap_arguments( &parser ); ++argind )
     {
     const int code = ap_code( &parser, argind );
-    const char * const arg = ap_argument( &parser, argind );
     if( !code ) break;					/* no more options */
+    const char * const pn = ap_parsed_name( &parser, argind );
+    const char * const arg = ap_argument( &parser, argind );
     switch( code )
       {
       case 'a': ignore_trailing = false; break;
@@ -761,7 +797,7 @@ int main( const int argc, const char * const argv[] )
                 else { default_output_filename = arg; } break;
       case 'q': verbosity = -1; break;
       case 't': set_mode( &program_mode, m_test ); break;
-      case 'u': buffer_size = get_dict_size( arg ); break;
+      case 'u': buffer_size = get_dict_size( arg, pn ); break;
       case 'v': if( verbosity < 4 ) ++verbosity; break;
       case 'V': show_version(); return 0;
       case opt_lt: loose_trailing = true; break;
@@ -769,15 +805,17 @@ int main( const int argc, const char * const argv[] )
       }
     } /* end process options */
 
-#if defined(__MSVCRT__) || defined(__OS2__) || defined(__DJGPP__)
+#if defined __MSVCRT__ || defined __OS2__ || defined __DJGPP__
   setmode( STDIN_FILENO, O_BINARY );
   setmode( STDOUT_FILENO, O_BINARY );
 #endif
 
-  num_filenames = max( 1, ap_arguments( &parser ) - argind );
+  static const char ** filenames = 0;
+  int num_filenames = max( 1, ap_arguments( &parser ) - argind );
   filenames = resize_buffer( filenames, num_filenames * sizeof filenames[0] );
   filenames[0] = "-";
 
+  bool filenames_given = false;
   for( i = 0; argind + i < ap_arguments( &parser ); ++i )
     {
     filenames[i] = ap_argument( &parser, argind + i );
@@ -816,17 +854,18 @@ int main( const int argc, const char * const argv[] )
   if( !to_stdout && program_mode != m_test && ( filenames_given || to_file ) )
     set_signals( signal_handler );
 
+  static struct Pretty_print pp;
   Pp_init( &pp, filenames, num_filenames );
 
+  int failed_tests = 0;
+  int retval = 0;
   const bool one_to_one = !to_stdout && program_mode != m_test && !to_file;
+  bool stdin_used = false;
   for( i = 0; i < num_filenames; ++i )
     {
-    unsigned long long cfile_size;
     const char * input_filename = "";
     int infd;
-    int tmp;
     struct stat in_stats;
-    const struct stat * in_statsp;
 
     Pp_set_name( &pp, filenames[i] );
     if( strcmp( filenames[i], "-" ) == 0 )
@@ -872,11 +911,13 @@ int main( const int argc, const char * const argv[] )
         }
       }
 
-    in_statsp = ( input_filename[0] && one_to_one ) ? &in_stats : 0;
-    cfile_size = ( input_filename[0] && S_ISREG( in_stats.st_mode ) ) ?
-      ( in_stats.st_size + 99 ) / 100 : 0;
-    tmp = decompress( cfile_size, infd, &pp, buffer_size, ignore_trailing,
-                      loose_trailing, program_mode == m_test );
+    const struct stat * const in_statsp =
+      ( input_filename[0] && one_to_one ) ? &in_stats : 0;
+    const unsigned long long cfile_size =
+      ( input_filename[0] && S_ISREG( in_stats.st_mode ) ) ?
+        ( in_stats.st_size + 99 ) / 100 : 0;
+    int tmp = decompress( cfile_size, infd, &pp, buffer_size, ignore_trailing,
+                          loose_trailing, program_mode == m_test );
     if( close( infd ) != 0 )
       { show_file_error( pp.name, "Error closing input file", errno );
         set_retval( &tmp, 1 ); }
